from abc import ABCMeta, abstractmethod
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from models import Module


class EventHandler(metaclass=ABCMeta):
    @abstractmethod
    def init(self, api: 'Module', dto: Any) -> None:
        pass
    @abstractmethod
    def subscribe(self) -> None:
        pass
    @abstractmethod
    def OnReceived(self) -> None:
        pass


from typing import Any, Union

from utils.win32client import dispatch


class Module:
    __module: Any

    def __init__(self, name: str) -> None:
        self.__module = dispatch(name)

    def get_header_value(self, type: int) -> Union[int, str, float]:
        return self.__module.GetHeaderValue(type)

    def set_input_value(self, type: int, code: str):
        self.__module.SetInputValue(type, code)

    def subscribe(self):
        self.__module.Subscribe()

    def unsubscribe(self):
        self.__module.Unsubscribe()


from typing import TYPE_CHECKING, Union, Any

from utils import dispatch
from models.creon import Module


class CreonAPI:
    __module: Union['Module', Any]

    def __init__(self, name: str) -> None:
        self.__module = Module(name)


import json

from constants import STOCK_CUR
from models.live_price import LivePrice, LivePriceHandler
from models.creon import CreonAPI
from utils import bind


class LivePriceAPI(CreonAPI):
    __handler: LivePriceHandler
    def __init__(self, code: str) -> None:
        super().__init__(STOCK_CUR)
        self.__module.set_input_value(0, code)
        self.dto = LivePrice(code=code, name='', price=0, time=0)

    @property
    def message(self) -> str:
        """json message from dto"""
        return json.dumps(self.dto.__dict__, ensure_ascii=False)

    def subscribe(self):
        self.__handler = bind(self.__module, LivePriceHandler)
        self.__handler.init(self.__module, self.dto)
        self.__handler.subscribe()

    async def unsubscribe(self):
        self.__module.unsubscribe()



from setuptools import setup, find_packages

setup(
    name                          = 'creon-plus',
    version                       = '0.0.1',
    description                   = '대신증권 Creon Plus API',
    long_description              = open('README.md').read(),
    long_description_content_type = 'text/markdown',
    author                        = 'jihogrammer',
    author_email                  = 'jihogrammer@gmail.com',
    license                       = 'MIT',
    packages                      = find_packages(),
    install_requires              = ['pywin32'],
    classifiers                   = [
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows"
    ],
)



from typing import TYPE_CHECKING
from models import EventHandler

if TYPE_CHECKING:
    from models import Module
    from models.live_price import LivePrice

class LivePriceHandler(EventHandler):
    def init(self, module: 'Module', dto: 'LivePrice') -> None:
        self.__module = module
        self.__dto = dto

    def subscribe(self) -> None:
        self.__module.subscribe()

    def OnReceived(self):
        self.__dto.code  = self.__module.get_header_value(0)
        self.__dto.name  = self.__module.get_header_value(1)
        self.__dto.price = self.__module.get_header_value(13)
        self.__dto.time  = self.__module.get_header_value(18)


from typing import TYPE_CHECKING
import win32com.client

if TYPE_CHECKING:
    from models import Module, EventHandler


def dispatch(name: str):
    return win32com.client.Dispatch(name)

def bind(module: 'Module', handler: 'EventHandler') -> 'EventHandler':
    return win32com.client.WithEvents(module, handler)